C*************************************************************
C     INVMTX : CALCULATION OF INVERSE MATRIX                 *
C*************************************************************
C
      SUBROUTINE INVMTX(NDATA,V)
      INCLUDE 'param.f'
      REAL*8 V(MX)
      INTEGER NDATA,ICOND
      ICOND=0
      IF(NDATA.LT.1) THEN
         WRITE(6,*) 'MATRIX DIMENSION < 1'
         STOP 9999
      END IF
      IF(NDATA.EQ.1) THEN 
         IF(V(1).NE.0.0) THEN
            V(1)=1.0/V(1)
         END IF
      ELSE
         CALL CHOLSK(NDATA,V,ICOND)
         IF(ICOND.EQ.-1) THEN
            WRITE(6,*) 'MATRIX NOT POSITIV'
            STOP 9999
         ELSE IF(ICOND.EQ.-2) THEN 
            WRITE(6,*) 'PIVOT ZERO'
            STOP 9999
         END IF
         CALL INVERS(NDATA,V,ICOND)
         IF(ICOND.EQ.-1) THEN
            WRITE(6,*) 'MATRIX SINGULAR'
            STOP 9999
         END IF
      END IF
      RETURN
      END
C
C*************************************************************
C     CHOLSK : LDU DECOMPOSITION BY MODIFIED CHOLESKI METHOD *
C              L TRIANGULAR MATRIX                           *
C              D INVERSE OF DIAGONAL MATRIX                  *
C              U TRANSPOSE OF L                              *
C*************************************************************
C
      SUBROUTINE CHOLSK(N,V,ICOND)
      INCLUDE 'param.f'
      REAL*8 V(MX)
      REAL*8 X1,XX,EPS
      INTEGER N,ICOND
      INTEGER I,J,IV,IJ,IK,JK,JJ
C
      EPS=1.0E-72
      ICOND=0
      If(V(1).EQ.0.0) THEN
         ICOND=-2
         GOTO 1000
      ELSE IF(V(1).LT.0.0) THEN
         ICOND=-1
         GOTO 1000
      END IF
C
      V(1)=1.0/V(1)
      X1  =V(3)-V(1)*V(2)*V(2)
      V(2)=V(1)*V(2)
      IF( DABS(X1).LT.DABS(V(3)*EPS) ) THEN
         ICOND=-2
         GO TO 1000
      END IF
      IF(X1.LT.0.0) THEN
         ICOND=-1
         GO TO 1000
      END IF
      V(3)=1.0/X1
C***************************************
C  FOR N=2                             *
C                                      *
C  V1  V2       1 /V1  V2/ V1          *
C  V2  V3  -->  V2/V1  V1/(V1*V3-V2*V2)*
C                                      *
C***************************************
      IF (N.EQ.2) GO TO 1000
C
      IV=4
      DO 10 I=1,N-2
         JK=2
         IK=IV
         DO 20,J=1,I
            XX=0.0
            DO 30 IJ=IV,IV+J-1
               XX=XX+V(IJ)*V(JK)
               JK=JK+1
   30       CONTINUE
            JK=JK+1
            IK=IK+1
            V(IK)=V(IK)-XX
   20    CONTINUE
         JJ=1
         IJ=IV
         XX=0.0
         DO 50 J=2,I+2
            X1=V(IJ)*V(JJ)
            XX=XX+V(IJ)*X1
            V(IJ)=X1
            JJ=JJ+J
            IJ=IJ+1
   50    CONTINUE
         X1=V(JJ)-XX
         IF( DABS(X1) .LT. DABS(V(JJ)*EPS) ) THEN
           ICOND=-2
           GOTO 1000
         END IF
         IF(X1.LT.0.0) THEN
           ICOND=-1
           GOTO 1000
         END IF
         V(JJ)=1.0/X1
         IV=JJ+1
   10 CONTINUE
 1000 RETURN
      END
C
C*************************************************************
C     INVERS : INVERSION OF LDU DECOMPOSED MATRIX            *
C              V=LDU                                         *
C              V**(-1)= U**(-1) D L**(-1)                    *
C*************************************************************
C
      SUBROUTINE INVERS(N,V,ICOND)
      INCLUDE 'param.f'
      REAL*8 V(MX)
      REAL*8 X1,XX
      INTEGER N,ICOND
      INTEGER I,J,K,IJ,IK,KJ,IJ1,IJ2,KJ1,KJ2,KI,II,IJSP
C
      ICOND=0
      V(2)=-V(2)
      IF(N.NE.2) THEN
         IJ2= 2
         DO 10,I=2,N-1
            IJ1=IJ2+2
            IJ2=IJ2+I+1
            KJ1=0
            DO 20 J=2,I
               XX =V(IJ1)
               IJ1 =IJ1+1
               KJ2 =J
               KJ1 =KJ1+KJ2
               KJ  =KJ1
               DO 30 IK=IJ1,IJ2
                  XX =XX+V(IK)*V(KJ)
                  KJ  =KJ+KJ2
                  KJ2 =KJ2+1
   30          CONTINUE
               V(KJ)=-XX
   20       CONTINUE
            V(IJ2)=-V(IJ2)
   10    CONTINUE
      END IF
C
      II= 1
      KI= 2
      XX=V(1)
      DO 40 K=2,N
         II=II+K
         IF(V(II).LT.0.0) THEN
            ICOND=-1
            GO TO 1000
         END IF
         X1=V(KI)*V(II)
         XX=XX+V(KI)*X1
         V(KI)=X1
         KI=KI+K
   40 CONTINUE
      V(1)=XX
      IF(N.EQ.2) GO TO 1000
      IJ=1
      DO 50 I=2,N-1
         DO 60 J=2,I
            IJ=IJ+1
            KJ=IJ
            XX=V(KJ)
            IJSP=I+1-J
            DO 70 K=I,N-1
               KJ=KJ+K
               KI=KJ+IJSP
               XX=XX+V(KJ)*V(KI)
   70       CONTINUE
            V(IJ)=XX
   60    CONTINUE
         IJ=IJ+1
         KI=IJ+I
         II=IJ
         XX=V(II)
         DO 80 K=I+1,N
            II=II+K
            X1=V(KI)*V(II)
            XX=XX+V(KI)*X1
            V(KI)=X1
            KI=KI+K
   80    CONTINUE
         V(IJ)=XX
   50 CONTINUE
 1000 RETURN
      END
