C*********************************************************************
C*     PROGRAM SOK : SIMULTANEOUS EVALUATION BASED ON KALMAN3.1      *
C*                                       KYUSHU UNIVERSITY 1999      *
C*                                                                   *
C*                          ORIGINAL VERSION FEB. 1999 T.KAWANO      *
C*                          Mod.             Nov. 2020 N.OTSUKA      *
C*********************************************************************
      INCLUDE 'param.f'
C               NP : TOTAL NUMBER OF PARAMETERS
C               NX : MAXIMAL NUMBER OF EXPERIMENTAL DATA
C               NR : MAXIMAL NUMBER OF REACTION TYPES
C               MP : SIZE OF COVARIANCE OF PARAMETERS = NP*(NP+1)/2
C               MX : SIZE OF COVARIANCE OF DATA       = NX*(NX+1)/2
C
      INTEGER NW,MR,NO
      PARAMETER(NW=100,MR=45)
C               NW : MAXIMAL NUMBER OF MEASUREMENTS IN A REACTION
C               MR : NUMBER OF RATIO MEASUREMENTS = NR*(NR-1)/2
C
      PARAMETER(NO=1)
C     PARAMETER(NO=0)
C               NO : ORDER OF POLYNOMIAL = 1
C
      REAL*8 X(NX),Y(NX),Z(NX),V(MX)
      REAL*8 E0(NP),D(NX,NP)
      REAL*8 P0(NP),P1(NP),PE(NP),X0(MP)
      REAL*8 EW(NW),CHISQ,CHITOT,RATIO,EMIN0,EMAX0,EMINIM,EMAXIM
      REAL*8 X1,X2
      INTEGER NENRG(NR),INDEX(NR),KRATIO(MR)
      INTEGER NDATA,NPARM,NREAC,NMSUR,NTOTAL,NRATIO,NKIND
      INTEGER KCOVEX,KCTL1,KCTL2,KCTL3
      INTEGER ISEQNO,IREAC,IMSUR,IR,JR,IDR,I,J,ISTART
      CHARACTER*12 PNAME(NR)
      CHARACTER*80 TITLE
      real*8 cw(nw)
      integer kprm(nw)
C
      DATA P0,P1,EW/ NP*0.0,NP*0.0,NW*1.0/
      DATA EMIN0,EMAX0,EMINIM,EMAXIM/ 4*0.0/
      DATA NTOTAL,ISEQNO,KCOVEX,NRATIO,CHITOT/ 0, 0, 0, 0, 0.0/
      DATA PNAME / NR*'            '/
      data cw/nw*1.0/
C
      READ(5,2000) TITLE
      READ(5,2015) NREAC,KCOVEX,KCTL1,KCTL2,KCTL3,EMIN0,EMAX0
C
      CALL GETPRM(KCTL1,NPARM,NKIND,NENRG,E0,P0,P1,PE,X0,PNAME,INDEX)
c     IF(KCTL3 .NE. 0) THEN
c        CALL RSTART(KCTL3,NPARM,NKIND,NENRG,E0,P0,P1,PE,X0,PNAME,INDEX)
c     END IF
C
      WRITE(6,2000) TITLE
      WRITE(6,2040) NKIND
      WRITE(6,2050) NPARM
C
      EMINIM=EMIN0
      EMAXIM=EMAX0
C
C***************************************
C     REACTION LOOP                    *
C***************************************
      DO 1000 IREAC=1,NREAC
         READ(5,2010,END=1010) IDR,NMSUR
         do i=1,nmsur
           read(5,2021) ew(i), cw(i), kprm(i)
         enddo
 2021 format(2E11.4,i10)
CO       READ(5,2020) (EW(I),I=1,NMSUR)
CD       WRITE(6,2020)(EW(I),I=1,NMSUR)
         IR=IDR/100
         JR=MOD(IDR,100)
         IF(IR.NE.0) THEN
            NRATIO=NRATIO+1
            KRATIO(NRATIO)=IDR
            IF(EMIN0 .EQ. 0.0) THEN
               X1=E0(INDEX(IR)+1)
               X2=E0(INDEX(JR)+1)
               EMINIM=MAX(X1,X2)
            END IF
            IF(EMAX0 .EQ. 0.0) THEN
               X1=E0(INDEX(IR)+NENRG(IR))
               X2=E0(INDEX(JR)+NENRG(JR))
               EMAXIM=MIN(X1,X2)
            END IF
         ELSE
            IF(EMIN0 .EQ. 0.0) EMINIM=E0(INDEX(JR)+1)
            IF(EMAX0 .EQ. 0.0) EMAXIM=E0(INDEX(JR)+NENRG(JR))
         END IF
C
C***************************************
C     MEASUREMENT LOOP                 *
C***************************************
         DO 1100 IMSUR=1,NMSUR
            CALL EXDATA(X,Y,Z,V,EMAXIM,EMINIM,
     +                  EW(IMSUR),cw(imsur),NDATA,1,KCOVEX)
            CALL OUTEXP(X,Y,Z,NDATA,IREAC,ISEQNO,IMSUR)
            IF(EW(IMSUR).EQ.0.0 .OR. NDATA.EQ.0) THEN
               CHISQ=0.0
               GO TO 1200
            END IF
C 
C***************************************
C     MAIN CALCULATION                 *
C***************************************
            IF(IR.EQ.0) THEN
              CALL DESGN1(NO,D,X,E0,NDATA,NENRG(JR),INDEX(JR))
            ELSE
              CALL DESGN2(NO,D,X,E0,NDATA,NENRG(IR),INDEX(IR),
     &                                    NENRG(JR),INDEX(JR))
            END IF
            if (kprm(imsur).gt.0) then
              call desgn4(d,ndata,index(nkind),kprm(imsur))
            endif
            IF(KCTL1.EQ.0) THEN
               CALL PRCALC(NDATA,NENRG(JR),NPARM,X,Y,Z,V,P1,X0,E0,D)
            ELSE
               CALL INVMTX(NDATA,V)
            END IF
            CALL CHISQR(NDATA,NPARM,Y,Z,V,P1,D,CHISQ)
C
            CHITOT=CHITOT+CHISQ
            NTOTAL=NTOTAL+NDATA
C
 1200       CONTINUE
            IF(IR.EQ.0) THEN
               WRITE(6,2080) PNAME(JR)
            ELSE
               WRITE(6,2085) PNAME(IR),PNAME(JR)
            END IF
            WRITE(6,2090) ISEQNO+IMSUR,IMSUR,NDATA,NTOTAL
            WRITE(6,2100) CHISQ,CHITOT
C
CD          WRITE(0,2090) ISEQNO+IMSUR,IMSUR,NDATA,NTOTAL
C
 1100    CONTINUE
C
         ISEQNO=ISEQNO+IMSUR-1
 1000 CONTINUE
 1010 CONTINUE
C
C***************************************
C     OUTPUT POSTERIOR PARAMETERS      *
C***************************************
      IF( (NTOTAL-NPARM).LE.0) THEN
          RATIO=0.0
      ELSE
          RATIO=CHITOT/DBLE(NTOTAL-NPARM)
      END IF
      WRITE(6,2110) CHITOT,NTOTAL-NPARM,RATIO
      IF(RATIO.LE.1.0) RATIO=1.0
      RATIO=DSQRT(RATIO)
C
      WRITE(6,2120)
      DO 100 I=1,NPARM
  100    PE(I)=DSQRT(X0(I*(I+1)/2))
      DO 110 I=1,NPARM
         ISTART=I*(I-1)/2
         DO 110 J=1,I
  110       X0(ISTART+J)=X0(ISTART+J)/PE(I)/PE(J)
      DO 120 I=1,NPARM
C        WRITE(6,2140) I,E0(I),EXP(P0(I)),EXP(P1(I)),PE(I)*100.0
         WRITE(6,2140) I,E0(I),EXP(P0(I)),EXP(P1(I)),
     &                                    PE(I)*RATIO*100.0
  120 CONTINUE
C
C***************************************
C     WRITE RESULTS ON FILE            *
C***************************************
      IF(KCTL1.EQ.0) THEN
         CALL OUTSPL(NPARM,NKIND,NENRG,INDEX,E0,P0,P1,PE,X0,PNAME)
         CALL SRATIO(NO,NPARM,NKIND,NENRG,INDEX,E0,P0,P1,PE,PNAME,
     &               NRATIO,KRATIO,D)
      END IF
C
      DO 130 I=1,NPARM
         E0(I)=E0(I)
         P0(I)=EXP(P0(I))
         P1(I)=EXP(P1(I))
  130 CONTINUE
C
      IF(KCTL1.EQ.0) THEN
         CALL CVRSPL(NPARM,NKIND,NENRG,INDEX,E0,P1,PE,X0,PNAME)
      END IF
      IF(KCTL2.EQ.1) THEN
         CALL SPLINE(NPARM,NKIND,NENRG,INDEX,E0,P1,PNAME)
      END IF
C
C
 2000 FORMAT(A80)
 2010 FORMAT(14I5)
 2015 FORMAT(5I5,5X,2E10.3)
 2020 FORMAT(7E10.3)
 2030 FORMAT(6(1PE11.4))
 2040 FORMAT('NUMBER OF DATA BLOCKS  ',I5 )
 2050 FORMAT('NUMBER OF SPLINE KNOTS ',I5/)
 2060 FORMAT(15X,I5,23X,I5)
 2080 FORMAT('CROSS SECTION :',A12)
 2085 FORMAT('CROSS SECTION :',A12,A12)
 2090 FORMAT('SQ.NO.:',I5,3X,
     1       'REACT.:',I5,3X,
     2       'POINTS:',I5,3X,
     3       'SUM UP:',I5)
 2100 FORMAT('PARTIAL CHI SQ:',1PE12.5,3X,'CUMULATIVE :',1PE12.5/)
 2110 FORMAT('CHI-SQUARE TEST !   ',   15X,'CHI - S = ',1PE12.5,
     &      /'DEGREE OF FREEDOM = ',I5,10X,'RATIO   = ',1PE12.5)
 2120 FORMAT(/5X,' PARAMETER  ',
     &'  INITIAL  ','  FINAL    ','  ERROR    '/)
 2130 FORMAT(I5,1X,1PE11.4,3(1PE11.4),1X,'(ABS)')
 2140 FORMAT(I5,1X,1PE11.4,3(1PE11.4),1X,'( % )')
      STOP
      END
